<?php

namespace Spec\Infomaker\Everyware\Support;

use Infomaker\Everyware\Support\Collection;
use Infomaker\Everyware\Support\Date;
use Infomaker\Everyware\Support\GenericPropertyObject;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class GenericPropertyObjectSpec extends ObjectBehavior
{
    public function it_is_initializable()
    {
        $this->shouldHaveType(GenericPropertyObject::class);
    }

    public function it_should_support_set_for_multible_properties()
    {
        $this->fill([
            'name' => 'John Doe',
            'age' => 30
        ]);

        $this->name->shouldReturn('John Doe');
        $this->age->shouldReturn(30);
    }

    public function it_should_have_magic_getter_and_setter()
    {
        $set_value = 'value';

        $this->missing_property->shouldReturn(null);
        $this->missing_property = $set_value;
        $this->get('missing_property')->shouldReturn($set_value);
        $this->missing_property->shouldReturn($set_value);
    }

    public function it_should_have_support_default_values_for_properties()
    {

        $this->get('missing_property', 'default_string')->shouldReturn('default_string');

    }

    public function it_should_have_support_for_closure_properties()
    {

        $this->set('method', $closure = function () {
            return 'return value';
        });

        $this->method->shouldReturn($closure);

    }

    public function it_can_return_single_value()
    {
        $set_value = 'return value';
        $this->value = [
            $set_value
        ];

        $this->value->shouldReturn((array)$set_value);
        $this->getSingleValue('value')->shouldReturn($set_value);
    }

    public function it_can_return_multi_value()
    {
        $set_value = 'return value';
        $empty_value = '';
        $false_value = false;
        $null_value = null;

        $this->value = $set_value;
        $this->false = $false_value;
        $this->empty = $empty_value;
        $this->null = $null_value;

        $this->value->shouldReturn($set_value);
        $this->getMultiValue('value')->shouldReturn((array)$set_value);
        $this->getMultiValue('false')->shouldReturn((array)$false_value);
        $this->getMultiValue('empty')->shouldReturn((array)$empty_value);
        $this->getMultiValue('null')->shouldReturn((array)$null_value);
    }

    public function it_can_return_bool_value()
    {
        $set_value = 'true';

        $this->value = $set_value;
        $this->multi_value = [
            'false'
        ];
        $this->getBoolValue('value')->shouldReturn(true);
        $this->getBoolValue('multi_value')->shouldReturn(false);
    }

    public function it_can_return_date_class_from_oc_date_string()
    {
        $set_value = '2018-10-08T08:48:20Z';
        $this->value = $set_value;
        $this->multi_value = [
            'false'
        ];
        $this->getDateValue('value')->shouldReturnAnInstanceOf(Date::class);
        $this->getDateValue('multi_value')->shouldReturnAnInstanceOf(Date::class);
    }

    public function it_offers_fallback_fetching_of_single_value()
    {
        $first_value = 'first return value';
        $second_value = 'second return value';
        $third_value = 'third return value';
        $empty_value = '';
        $false_value = false;
        $null_value = null;

        $this->fill([
            'first' => $first_value,
            'second' => $second_value,
            'third' => $third_value,
            'empty' => $empty_value,
            'false' => $false_value,
            'null' => $null_value
        ]);

        $this->getFirstOf('unknown,second,first')->shouldReturn($second_value);
        $this->getFirstOf('empty,first')->shouldReturn($first_value);
        $this->getFirstOf('false,third')->shouldReturn($third_value);
        $this->getFirstOf('null,false,third')->shouldReturn($third_value);
    }

    public function it_offers_fallback_fetching_as_multi_value()
    {
        $first_value = ['first return value'];
        $second_value = 'second return value';
        $empty_value = [];

        $this->fill([
            'first' => $first_value,
            'second' => $second_value,
            'empty' => $empty_value
        ]);

        $this->getFirstOf('unknown,second,first', false)->shouldReturn((array)$second_value);
        $this->getFirstOf('empty,first', false)->shouldReturn($first_value);
    }

    public function it_should_include_empty_values_in_fallback_multi_value()
    {
        $set_value = 'return value';
        $empty_value = '';
        $false_value = false;
        $null_value = null;

        $this->fill([
            'value' => $set_value,
            'empty' => $empty_value,
            'false' => $false_value,
            'null' => $null_value
        ]);

        $this->getFirstOf('unknown,value', false)->shouldReturn((array)$set_value);
        $this->getFirstOf('empty,value', false)->shouldReturn((array)$empty_value);
        $this->getFirstOf('false,value', false)->shouldReturn((array)$false_value);
        $this->getFirstOf('null,false,value', false)->shouldReturn((array)$false_value);
    }

    public function it_can_return_all_properties_as_array()
    {
        $set_value = 'return value';
        $empty_value = '';
        $false_value = false;
        $null_value = null;

        $properties = [
            'value' => $set_value,
            'empty' => $empty_value,
            'false' => $false_value,
            'null' => $null_value
        ];

        $this->fill($properties);

        $this->getProperties()->shouldReturn($properties);
    }

    public function it_can_return_all_properties_as_json()
    {
        $set_value = 'return value';
        $empty_value = '';
        $false_value = false;
        $null_value = null;

        $properties = [
            'value' => $set_value,
            'empty' => $empty_value,
            'false' => $false_value,
            'null' => $null_value
        ];

        $this->fill($properties);

        $this->toJson()->shouldReturn(json_encode($properties));
    }

    public function it_can_iterate_self_when_returning_all_properties_as_array()
    {
        $property_object = new GenericPropertyObject();
        $set_value = 'return value';
        $empty_value = '';
        $false_value = false;
        $null_value = null;

        $properties = [
            'value' => $set_value,
            'empty' => $empty_value,
            'false' => $false_value,
            'null' => $null_value
        ];

        $property_object->fill($properties);

        $this->fill([
            'value' => $set_value,
            'empty' => $empty_value,
            'false' => $false_value,
            'null' => $null_value,
            'property_object' => $property_object
        ]);

        $this->toArray()->shouldReturn([
            'value' => $set_value,
            'empty' => $empty_value,
            'false' => $false_value,
            'null' => $null_value,
            'property_object' => $properties
        ]);
    }

    public function it_has_support_for_iterating_collections_when_returning_all_properties_as_array()
    {
        $set_value = 'return value';
        $empty_value = '';
        $false_value = false;
        $null_value = null;
        $property_object = new GenericPropertyObject();

        $properties = [
            'value' => $set_value,
            'empty' => $empty_value,
            'false' => $false_value,
            'null' => $null_value
        ];

        $property_object->fill($properties);

        $this->fill([
            'value' => $set_value,
            'empty' => $empty_value,
            'false' => $false_value,
            'null' => $null_value,
            'property_object_collection' => new Collection([
                $property_object,
                $property_object,
                $property_object
            ])
        ]);

        $this->toArray()->shouldReturn([
            'value' => $set_value,
            'empty' => $empty_value,
            'false' => $false_value,
            'null' => $null_value,
            'property_object_collection' => [
                $properties,
                $properties,
                $properties
            ]
        ]);
    }
}
